function [x,u,J,converg,iterat] = NLloop_NashOL(fname,NP,NU,x,xSm1,u,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,S,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,MC)
%running the nonlinearity loop for Nash open-loop solution until the algorithm converges, or the NLmaxit
%is achieved, or the results worsen in 3 runs in a row.
%in each iteration run backward: 1) calculate H_t , h_t: according to equations (110) and (111)
%2) Lambda_t: according to equation (109)
%After that the convergence of the algortihm is checked and the
%same procedure restarted again (if there is no convergence)

n=size(x,2);
m=size(u,2);

iterat = 0;
converg = 0;
diverg = 0;
divergence_info_on = 1; %if = 1 report (temporal) divergence

while ((converg < (n + m) * (T - S + 1)) && (iterat < NLmaxit) && diverg < 3)
    iterat = iterat + 1;
    if iterat==1
        Jiter_start = calculateJ(NP,x,u,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
    else
        Jiter_start = J;
    end
    J = 0;
    xold = x;
    uold = u;
    
    H=zeros(n,n,NP,T-S+1);
    h=zeros(n,NP,T-S+1);
    Lambda=zeros(n,n,T-S+1);
    eta=zeros(n,T-S+1);
    A=zeros(n,n,T-S+1);
    B=zeros(n,m,T-S+1);
    
    t=T;
    alpha_t=1/(alpha^(t-1)); %discount factor in t
    alpha_tm1=1/(alpha^(t-2)); %discount factor in t-1
    if t==TE %check for special time periods with different weights Wtxx, Wtuu
        Q=Wtxx;
        R=Wtuu;
    else
        Q=Wxx;
        R=Wuu;
    end
    x0t = x(t,:);
    x0tm1 = x(t-1,:);
    u0t = u(t,:);
    z0t = z(t,:);
    eps = zeros(n,1);
    %linearize system
    [A(:,:,t),B(:,:,t),c] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,eps);
    
    Lambda(:,:,t)=eye(n);
    eta(:,t) = c;
    nu=1;
    for player=1:NP
        nupi=nu+NU(player)-1; %nu:nupi control variables of player i
        H(:,:,player,t)=Q(:,:,player)*alpha_t;
        h(:,player,t)=-Q(:,:,player)*alpha_t*x_target(t,:,player)';
        Lambda(:,:,t)=Lambda(:,:,t)+B(:,player,t)*inv(R(nu:nupi,nu:nupi,player)*alpha_t)*(B(:,player,t)')*H(:,:,player,t);
        eta(:,t) = eta(:,t) + B(:,player,t)*(u_target(t,nu:nupi,player)'-inv(R(nu:nupi,nu:nupi,player)*alpha_t)*(B(:,player,t)')*h(:,player,t));
        nu=nu+NU(player);
    end
    %calculate Htm1
    for player=1:NP
        Lambda_inv=inv(Lambda(:,:,t));
        H(:,:,player,t-1)=Q(:,:,player)*alpha_tm1 + (A(:,:,t)')*H(:,:,player,t)*Lambda_inv*A(:,:,t);
        h(:,player,t-1)=-Q(:,:,player)*alpha_tm1*x_target(t-1,:,player)' + (A(:,:,t)')*(H(:,:,player,t)*Lambda_inv*eta(:,t)+h(:,player,t));
    end
    
    for t=T-1:-1:S+1
        alpha_t=1/(alpha^(t-1)); %discount factor in t
        alpha_tm1=1/(alpha^(t-2)); %discount factor in t-1
        if t==TE %check for special time periods with different weights Wtxx, Wtuu
            Q=Wtxx;
            R=Wtuu;
        else
            Q=Wxx;
            R=Wuu;
        end
        x0t = x(t,:);
        x0tm1 = x(t-1,:);
        u0t = u(t,:);
        z0t = z(t,:);
        eps = zeros(n,1);
        %linearize system
        [A(:,:,t),B(:,:,t),c] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,eps);
        
        
        Lambda(:,:,t)=eye(n);
        eta(:,t) = c;
        nu=1;
        for player=1:NP
            nupi=nu+NU(player)-1; %nu:nupi control variables of player i
            Lambda(:,:,t)=Lambda(:,:,t)+B(:,player,t)*inv(R(nu:nupi,nu:nupi,player)*alpha_t)*(B(:,player,t)')*H(:,:,player,t);
            eta(:,t) = eta(:,t) + B(:,player,t)*(u_target(t,nu:nupi,player)'-inv(R(nu:nupi,nu:nupi,player)*alpha_t)*(B(:,player,t)')*h(:,player,t));
            nu=nu+NU(player);
        end
        for player=1:NP
            Lambda_inv=inv(Lambda(:,:,t));
            H(:,:,player,t-1)=Q(:,:,player)*alpha_tm1 + (A(:,:,t)')*H(:,:,player,t)*Lambda_inv*A(:,:,t);
            h(:,player,t-1)=-Q(:,:,player)*alpha_tm1*x_target(t-1,:,player)' + (A(:,:,t)')*(H(:,:,player,t)*Lambda_inv*eta(:,t)+h(:,player,t));
        end
    end
    
    t=S;
    alpha_t=1/(alpha^(t-1)); %discount factor in t
    if t==TE %check for special time periods with different weights Wtxx, Wtuu
        Q=Wtxx;
        R=Wtuu;
    else
        Q=Wxx;
        R=Wuu;
    end
    x0t = x(t,:);
    x0tm1 = xSm1;
    u0t = u(t,:);
    z0t = z(t,:);
    eps = zeros(n,1);
    %linearize system
    [A(:,:,t),B(:,:,t),c] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,eps);
    
    Lambda(:,:,t)=eye(n);
    eta(:,t) = c;
    nu=1;
    for player=1:NP
        nupi=nu+NU(player)-1; %nu:nupi control variables of player i
        Lambda(:,:,t)=Lambda(:,:,t)+B(:,player,t)*inv(R(nu:nupi,nu:nupi,player)*alpha_t)*(B(:,player,t)')*H(:,:,player,t);
        eta(:,t) = eta(:,t) + B(:,player,t)*(u_target(t,nu:nupi,player)'-inv(R(nu:nupi,nu:nupi,player)*alpha_t)*(B(:,player,t)')*h(:,player,t));
        nu=nu+NU(player);
    end
    
    %'Forward loop': Compute new values for x and u
    
    % linear solution
    
%     while t <= T - S + 1
%         if t==1
%             x(t,:)=inv(Lambda(:,:,t))*(A(:,:,t)*xSm1' + eta(:,t));
%         else
%             x(t,:)=inv(Lambda(:,:,t))*(A(:,:,t)*x(t-1,:)' + eta(:,t));
%         end
%         nu=1;
%         for player=1:NP
%             nupi=nu+NU(player)-1; %nu:nupi control variables of player i
%             u(t,nu:nupi)=u_target(t,nu:nupi,player)-inv(R(nu:nupi,nu:nupi,player)/(alpha^(t-1)))*(B(:,player,t)')*(H(:,:,player,t)*x(t,:)'+h(:,player,t));
%             nu=nu+NU(player);
%         end
%         t = t + 1;
%     end

    while t <= T - S + 1
        if t==1
            x(t,:)=inv(Lambda(:,:,t))*(A(:,:,t)*xSm1' + eta(:,t));
        else
            x(t,:)=inv(Lambda(:,:,t))*(A(:,:,t)*x(t-1,:)' + eta(:,t));
        end
        nu=1;
        for player=1:NP
            nupi=nu+NU(player)-1; %nu:nupi control variables of player i
            u(t,nu:nupi)=u_target(t,nu:nupi,player)-inv(R(nu:nupi,nu:nupi,player)/(alpha^(t-1)))*(B(:,player,t)')*(H(:,:,player,t)*x(t,:)'+h(:,player,t));
            nu=nu+NU(player);
        end
        
        if t==1
            x0_t = xSm1;                        % Starting guess
            xl_t = xSm1;
        else
            x0_t = x(t-1,:);                    % Starting guess
            xl_t = x(t-1,:);
        end
        u_t = u(t,:);
        z_t = z(t,:);
        [x(t,:),res,iter,fval]=fsolution(Ssolver,fname,x0_t,xl_t,u_t,z_t,theta,eps,Nmaxit,GSmaxit,SStoler,disp_err,t,0);        
        
        t = t + 1;
    end
    
    %check the convergency
    converg = 0;
    xdiff = abs((x-xold)./xold);
    udiff = abs((u-uold)./uold);
    for i=1:(T-S+1)
        for j=1:n
            if xdiff(i,j) <= NLtoler
                converg = converg + 1;
            end
        end
        for jj=1:m
            if udiff(i,jj) <= NLtoler
                converg = converg + 1;
            end
        end
    end
    
    %J = calculateJ(NP,x,u,x_target,u_target,Q,R,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
    J = calculateJ(NP,x,u,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
    Jiter_diff=J-Jiter_start;
    if (Jiter_diff(NP+1))>0 && divergence_info_on == 1
        disp(['(temporal) divergence in NashOL strategy: iterat = ' num2str(iterat) '; Jiter_diff = ' num2str(Jiter_diff)])
    end
end