function [x,u,J,converg,iterat] = NLloop_StackOL(fname,NP,NU,x,xSm1,u,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,S,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,MC)
%running the nonlinearity loop for Stackelberg open-loop solution until the algorithm converges, or the NLmaxit
%is achieved, or the results worsen in 3 runs in a row.
%in each iteration run backward according to equations (134) and (155) and
%then forward.
%After that the convergence of the algortihm is checked and the
%same procedure restarted again (if there is no convergence)

n=size(x,2);
m=size(u,2);

iterat = 0;
converg = 0;
diverg = 0;
divergence_info_on = 1; %if = 1 report (temporal) divergence

while ((converg < (n + m) * (T - S + 1)) && (iterat < NLmaxit) && diverg < 3)
    iterat = iterat + 1;
    if iterat==1
        Jiter_start = calculateJ(NP,x,u,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
    else
        Jiter_start = J;
    end
    J = 0;
    xold = x;
    uold = u;
    
%     H=zeros(n,n,NP);
%     h=zeros(n,NP);
    A=zeros(n,n);
    B=zeros(n,m);
%     G=zeros(m,n,T - S + 1);
%     g=zeros(m,T - S + 1);
%     K=zeros(n,n,T - S + 1);
%     k=zeros(n,T - S + 1);
    L=zeros(n,n,NP);
    li=zeros(n,1);
    Mix=zeros(n,n,NP);
    Mimmu=zeros(n,n,NP,NP);
    mi=zeros(n,NP);
    Phix = zeros(n,n,T-S+1);
    phii = zeros(n,1,T-S+1);
    Phimmu = zeros(n,n,NP,T-S+1);
    Psiix=zeros(n,n,NP,T-S+1);
    psii=zeros(n,NP,T-S+1);
    Psiimmu=zeros(n,n,NP,NP,T-S+1);
    mui=zeros(n,NP,T-S+1);
    Pix=zeros(m,n,T-S+1);
    pi=zeros(1,n,T-S+1);
    Pimmu=zeros(m,n,NP,T-S+1);
    alphai=zeros(m,T-S+1);
    
    for t=T:-1:S
        alpha_t=1/(alpha^(t-1)); %discount factor in t
        alpha_tm1=1/(alpha^(t-2)); %discount factor in t-1
        if t==TE %check for special time periods with different weights Wtxx, Wtuu
            Qt=Wtxx*alpha_t;
            Rt=Wtuu*alpha_t;
        else
            Qt=Wxx*alpha_t;
            Rt=Wuu*alpha_t;
        end
        if t>S
            tm1=t-1;
            if tm1==TE %check for special time periods with different weights Wtxx, Wtuu
                Qtm1=Wtxx*alpha_tm1;
                %Rt=Wtuu*alpha_t;
            else
                Qtm1=Wxx*alpha_tm1;
                %Rt=Wuu*alpha_t;
            end
        end
        x0t = x(t,:);
        if t==1
            x0tm1 = xSm1;
        else
            x0tm1 = x(t-1,:);
        end
        u0t = u(t,:);
        z0t = z(t,:);
        eps = zeros(n,1);
        %linearize system
        [A(:,:),B(:,:),c] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,eps);
        
        if t==T
            %nu=1;
            for player=1:NP
                %nupi=nu+NU(player)-1; %nu:nupi control variables of player i
                H(:,:,player)=Qt(:,:,player);
                h(:,player)=Qt(:,:,player)*x_target(t,:,player)';
                Mix(:,:,player)=Qt(:,:,player);
                %nu=nu+NU(player);
            end
            L(:,:,1)=Qt(:,:,1);
        end
        
        factor_A_Nn=zeros(m,m);
        inv_Ri = zeros(m,m,NP);
        R1invRiBi = zeros(m,n);
        BiQipLiBi = zeros(m,m);
        BiQjpLjBj = zeros(m,m);
        nu=NU(1)+1;
        for player=2:NP 
            nupi=nu+NU(player)-1; %nu:nupi control variables of player i
            inv_Ri(nu:nupi,nu:nupi,player) = inv(Rt(nu:nupi,nu:nupi,player));
            R1invRiBi(nu:nupi,:) = Rt(nu:nupi,nu:nupi,1)*inv_Ri(nu:nupi,nu:nupi,player)*B(:,nu:nupi)';
            BiQipLiBi(nu:nupi,nu:nupi) = B(:,nu:nupi)'*(Qt(:,:,player)+L(:,:,player))*B(:,nu:nupi);
            nuj=NU(1)+1;
            for player_j=2:NP
                nujpj=nuj+NU(player_j)-1;
                if player_j ~= player
                    BiQjpLjBj(nu:nupi,nuj:nujpj) = B(:,nu:nupi)'*(Qt(:,:,player_j)+L(:,:,player_j))*B(:,nuj:nujpj);
                end
                nuj=nuj+NU(player_j);
            end
            factor_A_Nn(nu:nupi,nu:nupi)= BiQipLiBi(nu:nupi,nu:nupi) + Rt(nu:nupi,nu:nupi,player) - R1invRiBi(nu:nupi,:)*Mimmu(:,:,player,player)*B(:,nu:nupi);
            nu=nu+NU(player);
        end
        
        
        %Gauss-Seidel to solve equation 134
        iter = 0; Nix=zeros(m,n); Nix_old = Nix+1;
        while (norm(Nix-Nix_old) > 0.00001 &&  iter < GSmaxit) %
            Nix_old = Nix;
            nu=NU(1)+1;
            for player=2:NP
                nupi=nu+NU(player)-1; %nu:nupi control variables of player i
                factor_sumj=zeros(1,n);
                nuj=NU(1)+1;
                for player_j=2:NP
                    nujpj=nuj+NU(player_j)-1;
                    if player_j ~= player
                        factor_sumj=factor_sumj+(BiQjpLjBj(nu:nupi,nuj:nujpj)-R1invRiBi(nu:nupi,:)*Mimmu(:,:,player,player_j)*B(:,nuj:nujpj))*Nix(nuj:nujpj,:);
                    end
                    nuj=nuj+NU(player_j);
                end
                factor_b=R1invRiBi(nu:nupi,:)*Mix(:,:,player)-B(:,nu:nupi)'*L(:,:,1)-factor_sumj;
                Nix(nu:nupi,:)=factor_A_Nn(nu:nupi,nu:nupi)\factor_b;
                nu=nu+NU(player);
            end
            iter= iter + 1;
        end % while
        if iter == GSmaxit
            disp(['exit for gauss-seidel because maximal number of runs (: ' num2str(GSmaxit) ') was reached in time period ' num2str(t)])
        end
        
        %Gauss-Seidel to solve equation 135
        Nimmu=zeros(m,n,NP);
        for player_m=2:NP
            iter = 0; Nimmu_old = Nimmu(:,:,player_m)+1;
            while (norm(Nimmu(:,:,player_m)-Nimmu_old) > 0.00001 &&  iter < GSmaxit) %
                Nimmu_old = Nimmu(:,:,player_m);
                nu=NU(1)+1;
                for player=2:NP
                    nupi=nu+NU(player)-1; %nu:nupi control variables of player i
                    factor_sumj=zeros(1,n);
                    nuj=NU(1)+1;
                    for player_j=2:NP
                        nujpj=nuj+NU(player_j)-1;
                        if player_j ~= player
                            factor_sumj=factor_sumj+(BiQjpLjBj(nu:nupi,nuj:nujpj)-R1invRiBi(nu:nupi,:)*Mimmu(:,:,player,player_j)*B(:,nuj:nujpj))*Nimmu(nuj:nujpj,:,player_m);
                        end
                        nuj=nuj+NU(player_j);
                    end
                    factor_b=R1invRiBi(nu:nupi,:)*Mimmu(:,:,player,player_m)*A-B(:,nu:nupi)'*(L(:,:,player_m)+Qt(:,:,player_m))*A-factor_sumj;
                    Nimmu(nu:nupi,:,player_m)=factor_A_Nn(nu:nupi,nu:nupi)\factor_b;
                    nu=nu+NU(player);
                end
                iter= iter + 1;
            end % while
            if iter == GSmaxit
                disp(['exit for gauss-seidel because maximal number of runs (: ' num2str(GSmaxit) ') was reached in time period ' num2str(t)])
            end
        end
        
        %Gauss-Seidel to solve equation 136
        iter = 0; ni=zeros(m,1); ni_old = ni+1;
        while (norm(ni-ni_old) > 0.00001 &&  iter < GSmaxit) %
            ni_old = ni;
            nu=NU(1)+1;
            for player=2:NP
                nupi=nu+NU(player)-1; %nu:nupi control variables of player i
                factor_sumj=0;
                nuj=NU(1)+1;
                for player_j=2:NP
                    nujpj=nuj+NU(player_j)-1;
                    if player_j ~= player
                        factor_sumj=factor_sumj+(BiQjpLjBj(nu:nupi,nuj:nujpj)-R1invRiBi(nu:nupi,:)*Mimmu(:,:,player,player_j)*B(:,nuj:nujpj))*ni(nuj:nujpj);
                    end
                    nuj=nuj+NU(player_j);
                end
                factor_b=R1invRiBi(nu:nupi,:)*(mi(:,player)-Qt(:,:,player)*x_target(t,:,player)')+B(:,nu:nupi)'*(Qt(:,:,1)*x_target(t,:,player)'-li)-Rt(nu:nupi,nu:nupi,1)*(u_target(t,nu:nupi,player)'-u_target(t,nu:nupi,1)')-factor_sumj;
                ni(nu:nupi,1)=factor_A_Nn(nu:nupi,nu:nupi)\factor_b;
                nu=nu+NU(player);
            end
            iter= iter + 1;
        end % while
        if iter == GSmaxit
            disp(['exit for gauss-seidel because maximal number of runs (: ' num2str(GSmaxit) ') was reached in time period ' num2str(t)])
        end
        
        Tix=zeros(m,n);
        nu=NU(1)+1;
        for player=2:NP %equation 137
            nupi=nu+NU(player)-1; %nu:nupi control variables of player i
            sumj=0;
            nuj=NU(1)+1;
            for player_j=2:NP
                nujpj=nuj+NU(player_j)-1;
                sumj=sumj+Mimmu(:,:,player,player_j)*B(:,nuj:nujpj)*Nix(nuj:nujpj,:);
                nuj=nuj+NU(player_j);
            end
            Tix(nu:nupi,:) = -inv_Ri(nu:nupi,nu:nupi,player)*B(:,nu:nupi)'*(Mix(:,:,player)+sumj);
            nu=nu+NU(player);
        end
        
        Timmu=zeros(m,n,NP);
        for player_m=2:NP
            nu=NU(1)+1;
            for player=2:NP %equation 138
                nupi=nu+NU(player)-1; %nu:nupi control variables of player i
                sumj=0;
                nuj=NU(1)+1;
                for player_j=2:NP
                    nujpj=nuj+NU(player_j)-1;
                    sumj=sumj+Mimmu(:,:,player,player_j)*B(:,nuj:nujpj)*Nimmu(nuj:nujpj,:,player_m);
                    nuj=nuj+NU(player_j);
                end
                Timmu(nu:nupi,:,player_m) = -inv_Ri(nu:nupi,nu:nupi,player)*B(:,nu:nupi)'*(Mimmu(:,:,player,player_m)*A+sumj);
                nu=nu+NU(player);
            end
        end
        
        ti=zeros(m,1);
        nu=NU(1)+1;
        for player=2:NP %equation 139
            nupi=nu+NU(player)-1; %nu:nupi control variables of player i
            sumj=0;
            nuj=NU(1)+1;
            for player_j=2:NP
                nujpj=nuj+NU(player_j)-1;
                sumj=sumj+Mimmu(:,:,player,player_j)*B(:,nuj:nujpj)*ni(nuj:nujpj,1);
                nuj=nuj+NU(player_j);
            end
            ti(nu:nupi) = u_target(t,nu:nupi,player)-inv_Ri(nu:nupi,nu:nupi,player)*B(:,nu:nupi)'*(mi(:,player)+sumj-Qt(:,:,player)*x_target(t,:,player)');
            nu=nu+NU(player);
        end       
        
        %calculate Wx, Wmmu and wi; equations 140-142
        sumW1=0; sumW2=0; sumw1=0; sumw2=0;
        nuj=NU(1)+1;
        for player_j=2:NP
            nujpj=nuj+NU(player_j)-1;
            sumW1=sumW1+L(:,:,player_j)*B(:,nuj:nujpj)*Nix(nuj:nujpj,:);
            sumW2=sumW2+ B(:,1:NU(1))'*Qt(:,:,player_j)*B(:,nuj:nujpj)*Nix(nuj:nujpj,:);
            sumw1=sumw1+L(:,:,player_j)*B(:,nuj:nujpj)*ni(nuj:nujpj,1);
            sumw2=sumw2+ B(:,1:NU(1))'*Qt(:,:,player_j)*B(:,nuj:nujpj)*ni(nuj:nujpj,1);
            nuj=nuj+NU(player_j);
        end

        inv_Ri(1:NU(1),1:NU(1),1)=inv(Rt(1:NU(1),1:NU(1),1));
        Wx=-inv_Ri(1:NU(1),1:NU(1),1)*(B(:,1:NU(1))'*(L(:,:,1) + sumW1) + sumW2);
        wi=u_target(t,1:NU(1),1) - inv_Ri(1:NU(1),1:NU(1),1)*(B(:,1:NU(1))'*(li + sumw1 - Qt(:,:,1)*x_target(t,:,1)') + sumw2);
        Wmmu=zeros(1,n,NP);
        for player_m=2:NP
            sumW1=0; sumW2=0;
            nuj=NU(1)+1;
            for player_j=2:NP
                nujpj=nuj+NU(player_j)-1;
                sumW1=sumW1+L(:,:,player_j)*B(:,nuj:nujpj)*Nimmu(nuj:nujpj,:,player_m);
                sumW2=sumW2+ B(:,1:NU(1))'*Qt(:,:,player_j)*B(:,nuj:nujpj)*Nimmu(nuj:nujpj,:,player_m);
                nuj=nuj+NU(player_j);
            end
            Wmmu(:,:,player_m) = -inv_Ri(1:NU(1),1:NU(1),1)*(B(:,1:NU(1))'*(L(:,:,player_m)*A + sumW1 + Qt(:,:,player_m)*A) + sumW2);
        end

        %calculate Phix, Phimmu and phi; equations 143-145
        factor_Phi = inv(eye(n)-B(:,1:NU(1))*Wx - B(:,NU(1)+1:m)*Tix(NU(1)+1:m,:));
        Phix(:,:,t) = factor_Phi*A;
        phii(:,:,t) = factor_Phi*(B(:,1:NU(1))*wi + B(:,NU(1)+1:m)*ti(NU(1)+1:m,1) + c);
        Phimmu(:,:,:,t) = zeros(n,n,NP);
        for player_m=2:NP
            Phimmu(:,:,player_m,t)= factor_Phi*(B(:,1:NU(1))*Wmmu(:,:,player_m) + B(:,NU(1)+1:m)*Timmu(NU(1)+1:m,:,player_m));
        end

        %calculate Psix, Psimmu and psi; equations 146-149
        nu=NU(1)+1;
        for player=2:NP 
            nupi=nu+NU(player)-1; %nu:nupi control variables of player i
            Psiix(:,:,player,t)=B(:,nu:nupi)*Nix(nu:nupi,:)*Phix(:,:,t);
            psii(:,player,t)=B(:,nu:nupi)*(Nix(nu:nupi,:)*phii(:,:,t) + ni(nu:nupi,1));
            for player_m=2:NP
                if player == player_m
                    Psiimmu(:,:,player,player,t)=A + B(:,nu:nupi)*(Nix(nu:nupi,:)*Phimmu(:,:,player,t) +Nimmu(nu:nupi,:,player));
                else
                    Psiimmu(:,:,player,player_m,t)=B(:,nu:nupi)*(Nix(nu:nupi,:)*Phimmu(:,:,player_m,t) +Nimmu(nu:nupi,:,player_m));
                end
            end
            nu=nu+NU(player);
        end       
        
        %save the Mix, Mimmu and mi from t and calculate these for t-1 (equations 150-152)
        Mixtp1=Mix;
        Mimmutp1=Mimmu;
        mitp1=mi;
        for player=2:NP 
            sumM1=0; summ1=0;
            for player_j=2:NP
                sumM1=sumM1+Mimmutp1(:,:,player,player_j)*Psiix(:,:,player_j,t);
                summ1=summ1+Mimmutp1(:,:,player,player_j)*psii(:,player_j,t);
            end
            Mix(:,:,player)=Qtm1(:,:,player) + A'*(Mixtp1(:,:,player)*Phix(:,:,t) + sumM1);
            mi(:,player)=A'*(Mixtp1(:,:,player)*phii(:,:,t) + summ1 + mitp1(:,player) - Qt(:,:,player)*x_target(t,:,player)');
        end
        for player_m=2:NP
            for player=2:NP
                sumM1=0; 
                for player_j=2:NP
                    sumM1=sumM1+Mimmutp1(:,:,player,player_j)*Psiimmu(:,:,player_j,player_m,t);
                end
                Mimmu(:,:,player,player_m)=A'*(Mixtp1(:,:,player)*Phimmu(:,:,player_m,t) + sumM1);
            end
        end
        
        %save the L and li from t and calculate these for t-1 (equations 153-155)
        Ltp1=L;
        litp1=li;
        sumL1=0; suml1=0; sumL2=0; suml2=0;
        nuj=NU(1)+1;
        for player_j=2:NP
            nujpj=nuj+NU(player_j)-1;
            sumL1=sumL1+Ltp1(:,:,player_j)*Psiix(:,:,player_j,t);
            sumL2=sumL2+Qt(:,:,player_j)*B(:,nuj:nujpj)*Nix(nuj:nujpj,:)*Phix(:,:,t);
            suml1=suml1+Ltp1(:,:,player_j)*psii(:,player_j,t);
            suml2=suml2+Qt(:,:,player_j)*B(:,nuj:nujpj)*(Nix(nuj:nujpj,:)*phii(:,:,t) + ni(nuj:nujpj,:));
            nuj=nuj+NU(player_j);
        end
        L(:,:,1)=Qtm1(:,:,1) + A'*( Ltp1(:,:,1)*Phix(:,:,t) + sumL1 + sumL2);
        li(:,1)=A'*( Ltp1(:,:,1)*phii(:,:,t) + suml1 + litp1(:,1) - Qt(:,:,1)*x_target(t,:,1)' + suml2);
        for player=2:NP
            sumL1=0; sumL2=0;
            nuj=NU(1)+1;
            for player_j=2:NP
                nujpj=nuj+NU(player_j)-1;
                sumL1=sumL1+Ltp1(:,:,player_j)*Psiimmu(:,:,player_j,player,t);
                sumL2=sumL2+Qt(:,:,player_j)*B(:,nuj:nujpj)*(Nix(nuj:nujpj,:)*Phimmu(:,:,player,t) + Nimmu(nuj:nujpj,:,player));
                nuj=nuj+NU(player_j);
            end
            L(:,:,player)=A'*( Ltp1(:,:,1)*Phimmu(:,:,player,t) + sumL1 + Qt(:,:,player)*A + sumL2);
        end

        %calculate Pix, Pimmu and alpha (equations 157-162)
        Pix(1:NU(1),:,t)=Wx*Phix(:,:,t);
        Pix(NU(1)+1:m,:,t)=Tix(NU(1)+1:m,:)*Phix(:,:,t);
        alphai(1:NU(1),t)=Wx*phii(:,:,t) + wi;
        alphai(NU(1)+1:m,t)=Tix(NU(1)+1:m,:)*phii(:,:,t) + ti(NU(1)+1:m);
        for player=2:NP
            Pimmu(1:NU(1),:,player,t)=Wx*Phimmu(:,:,player,t) + Wmmu(:,:,player);
            Pimmu(NU(1)+1:m,:,player,t)=Tix(NU(1)+1:m,:)*Phimmu(:,:,player,t) + Timmu(NU(1)+1:m,:,player);
        end
    end
    
    
    %'Forward loop': Compute new values for x and u
    while t <= T - S + 1        
        if t==1
            for player=2:NP
                mui(:,player,t)=Psiix(:,:,player,t)*xSm1'+ psii(:,player,t);
            end
        else
            for player=2:NP
                summu=0;
                for player_j=2:NP
                    summu=summu+Psiimmu(:,:,player,player_j,t)*mui(:,player_j,t-1);
                end
                mui(:,player,t)=Psiix(:,:,player,t)*x(t-1,:)' + summu + psii(:,player,t);
            end
        end
        
    % linear solution       
%         if t==1
%             u(t,:)=Pix(:,:,t)*xSm1' + alphai(:,t);
%             x(t,:)=Phix(:,:,t)*xSm1' + phii(:,:,t);
%         else
%             sumu=0; sumx=0;
%             for player_j=2:NP
%                 sumu=sumu+Pimmu(:,:,player_j,t)*mui(:,player_j,t-1);
%                 sumx=sumx+Phimmu(:,:,player_j,t)*mui(:,player_j,t-1);
%             end
%             u(t,:)=Pix(:,:,t)*x(t-1,:)' + sumu + alphai(:,t);
%             x(t,:)=Phix(:,:,t)*x(t-1,:)' + sumx + phii(:,:,t);
%         end
        
%nonlinear solution        
        if t==1
            x0_t = xSm1;                        % Starting guess
            xl_t = xSm1;
            u(t,:)=Pix(:,:,t)*xSm1' + alphai(:,t);
        else
            x0_t = x(t-1,:);                    % Starting guess
            xl_t = x(t-1,:);
            sumu=0; 
            for player_j=2:NP
                sumu=sumu+Pimmu(:,:,player_j,t)*mui(:,player_j,t-1);
            end
            u(t,:)=Pix(:,:,t)*x(t-1,:)' + sumu + alphai(:,t);
        end
        u_t = u(t,:);
        z_t = z(t,:);
        [x(t,:),res,iter,fval]=fsolution(Ssolver,fname,x0_t,xl_t,u_t,z_t,theta,eps,Nmaxit,GSmaxit,SStoler,disp_err,t,0);
        
        
        t = t + 1;
    end
    
    %check the convergency
    converg = 0;
    xdiff = abs((x-xold)./xold);
    udiff = abs((u-uold)./uold);
    for i=1:(T-S+1)
        for j=1:n
            if xdiff(i,j) <= NLtoler
                converg = converg + 1;
            end
        end
        for jj=1:m
            if udiff(i,jj) <= NLtoler
                converg = converg + 1;
            end
        end
    end
    
    %J = calculateJ(NP,x,u,x_target,u_target,Q,R,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
    J = calculateJ(NP,x,u,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
    Jiter_diff=J-Jiter_start;
    if (Jiter_diff(NP+1))>0 && divergence_info_on == 1
        disp(['(temporal) divergence in StackOL strategy: iterat = ' num2str(iterat) '; Jiter_diff = ' num2str(Jiter_diff)])
    end
end