%% Differential Evolution function
% starting with u_DE,x_DE search for
% Huu (and indirectly for Hxx), which minimizes the optimum control problem.
% f is the system of (nonlinear) equations.
function [Hxx,Huu,fitC,Hu,bestv,BestS] = de(fname,x_DE,xSm1,u_full,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC,T,S,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE,Jkind,threshold_input,beta)

%% Inputs:
% 'fname' name(path) to the function for calculating f
% 'maxitH' maximal number of iterations
% tol an absolute tolerance limit for DE: norm(f)<tol means convergence
% 'c' constant for the size of population 'P=c*N' for DE (usually: 2 < c < 10)
% neigDE the range for the neighbourhood of start population for DE
% depends on the quality and absolute values of u0; 'good'
% u0 => small neigDE; 'small' x0 => large neigDE

% Output:
% Hxx results for the state varaibles
% Huu results for the control varaibles
% fitC FitnessValues

N = ul(player)-uf(player)+1; %number of control variables
n = size(x_DE,2); %number of state variables
P=Pop_DE*N+1;   % size of population (+ 1 because of x0)
F=F_DE;   % parameter for linear combination
piv=CR_DE; % parameter for cross-over (probability for bits of 1. parent)
PopConverg=zeros(DEmaxit,P); %check the convergence
test_numel=zeros(DEmaxit,4);
fitC=zeros(DEmaxit,P);

Hx=zeros(T-S+1,n,P);
HxOffspring=zeros(T-S+1,n,P);

%% Generate start population, i.e. uniform values from the interval [u0(1-neigDE), u0(1+neigDE)].
temp = rand(T-S+1,N,P);
Hu=u_full(:,uf(player):ul(player)); %in each MC run use u as 1. population member
neigDE=neigDE(:,uf(player):ul(player));
for p=1:(P-1)
    for i=1:N
        for t=1:T-S+1
            if strncmpi(fname, 'mumod',5) == 1
                Hu(t,i,p+1)=neigDE(1,i) + (neigDE(2,i)-neigDE(1,i)).*temp(t,i,p);  % definition of absolute deviation from 0
            else
                Hu(t,i,p+1)=(1-neigDE(i))*Hu(t,i) + 2*neigDE(i)*Hu(t,i).*temp(t,i,p);  % definition of % deviation from u0
            end
        end
    end
end
if strncmpi(fname, 'macrae',6) || strncmpi(fname, 'atopt',5) == 1
    for p=1:(P-1)
        for t=1:T-S+1
            Hu(t,i,p+1)=neigDE*temp(t,1,p);  % definition of deviation from u0 for
            %initial population (under 'neigDE' per cent)
        end
    end
end
%Huinitial=Hu;

% calculate paths for x of the start population
for p = 1:P %% random initial solution (using u)
    u_full(:,uf(player):ul(player))=Hu(:,:,p);
    iter=0;
    for t=1:T-S+1 %calculate the initial solution path
        if t==1
            x0_t = xSm1;                        % Starting guess
            xl_t = xSm1;
        else
            x0_t = Hx(t-1,:,p);                    % Starting guess
            xl_t = Hx(t-1,:,p);
        end
        u_t = u_full(t,:);
        z_t = z(t,:);
        if size(eps,2)>1
            eps_t = eps(:,t);
        else
            eps_t = eps(:,1);
        end
        if iter>=0
            [Hx(t,:,p),res,iter,fval]=fsolution(Ssolver,fname,x0_t,xl_t,u_t,z_t,theta,eps_t,Nmaxit,GSmaxit,SStoler,disp_err,t,0);
        end
    end
    fitC(1,p)=0;
    % fitness of u(p),x(p)
    if iter<0
        fitC(1,p)=NaN;
    else
        J_DE = calculateJ(NP,Hx(:,:,p),u_full,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,Jkind,threshold_input,beta,0,0);
        fitC(1,p) = J_DE(player);
    end
    %Hxinitial=Hx;
end
save_results=0;
if save_results == 1
    save initial_de -double
end
%% Main body including the generation loop with creating offspring,
%% crossover and selecting
SStolerchange = 0;
it=2;
fitC(it,:)=fitC(it-1,:);

while it<=DEmaxit && numel(find(fitC(it-1,:)<min(fitC(it-1,:))*DEconvergCrit(1)))<=DEconvergCrit(2)*P && numel(find(PopConverg(it-1,:)>DEconvergCrit(3)))<=DEconvergCrit(4)*P %Until x% of the population converge to nearly the same objective value or x% of the population does not improve for y periods
    %     if (it<=0.85*DEmaxit && SStolerchange == 0)
    %         SStoler = SStoler*10000;
    %         SStolerchange = 1;
    %     elseif (it >= DEmaxit/5 && SStolerchange == 1)
    %             SStoler = SStoler/1;
    %             SStolerchange = 2;
    %     elseif (it >= .85*DEmaxit &&SStolerchange == 2)
    %             SStoler = SStoler/10000;
    %             SStolerchange = 3;
    %     end
    for p = 1:P     %% create offspring for member p
        rule=randperm(P); % choose 3 random members
        if rule(1)==p
            m1=Hu(:,:,rule(4));
        else
            m1=Hu(:,:,rule(1));
        end
        if rule(2)==p
            m2=Hu(:,:,rule(5));
        else
            m2=Hu(:,:,rule(2));
        end
        if rule(3)==p
            m3=Hu(:,:,rule(6));
        else
            m3=Hu(:,:,rule(3));
        end
        p2 = m1 + F*(m2 - m3); %create a second parent for member p
        crossover = rand(1,N*(T-S+1));
        for t=1:(T-S+1) %cross-over
            for i=1:N
                if crossover((t-1)*N+i) > piv
                    pOffspring(t,i,p)=Hu(t,i,p);
                else
                    pOffspring(t,i,p)=p2(t,i);
                end
            end
        end
        
        u_full(:,uf(player):ul(player))=pOffspring(:,:,p);
        iter=0;
        for t=1:T-S+1 %calculate the solution path for offspring p
            if t==1
                x0_t = xSm1;                        % Starting guess
                xl_t = xSm1;
            else
                x0_t = HxOffspring(t-1,:,p);                    % Starting guess
                xl_t = HxOffspring(t-1,:,p);
            end
            u_t = u_full(t,:);
            z_t = z(t,:);
            if size(eps,2)>1
                eps_t = eps(:,t);
            else
                eps_t = eps(:,1);
            end
            if iter>=0
                [HxOffspring(t,:,p),res,iter,fval]=fsolution(Ssolver,fname,x0_t,xl_t,u_t,z_t,theta,eps_t,Nmaxit,GSmaxit,SStoler,disp_err,t,0);
            end
        end
        
        fitOffspring(p)=0;
        % fitness of offspring
        if iter<0
            fitOffspring(p)=NaN;
        else
            J_offspring = calculateJ(NP,HxOffspring(:,:,p),u_full,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,Jkind,threshold_input,beta,0,0);
            fitOffspring(p) = J_offspring(player);
        end
        %comparing 'parents' and 'offsprings'
        if fitOffspring(p) < fitC(it,p) || isnan(fitC(it,p))% survive
            iReplace(:,:,p) = pOffspring(:,:,p);
            fitC(it+1,p)=fitOffspring(p);
            PopConverg(it,p)=0;
        else
            iReplace(:,:,p) = Hu(:,:,p);
            fitC(it+1,p)=fitC(it,p);
            if it>1
                PopConverg(it,p)=PopConverg(it-1,p)+1;
            else
                PopConverg(it,p)=1;
            end
        end
    end
    Hu=iReplace;
    test_numel(it,1,1)=numel(find(fitC(it,:)<min(fitC(it,:))*1.00001));
    test_numel(it,2,1)=numel(find(fitC(it,:)<min(fitC(it,:))*1.000001));
    test_numel(it,3,1)=numel(find(fitC(it,:)<min(fitC(it,:))*1.0000001));
    test_numel(it,4,1)=numel(find(fitC(it,:)<min(fitC(it,:))*1.00000001));
    
    it=it+1;
end
disp_converg=1;
if disp_converg==1 && it<DEmaxit
    disp(['it (' num2str(it) ')<DEmaxit in "de" function.'])
end
bestv=min(fitC(it,:)); %find the best solution in the final population

if isnan(bestv)
    Huu=nan(T-S+1,N);
    Hxx=nan(T-S+1,n);
else
    position=find(fitC(it,:)==bestv);
    
    Huu=u_full;
    Huu(:,uf(player):ul(player))=iReplace(:,:,position(1));
    for t=1:T-S+1 %calculate the solution path for best x
        if t==1
            x0_t = xSm1;                        % Starting guess
            xl_t = xSm1;
        else
            x0_t = Hxx(t-1,:);                    % Starting guess
            xl_t = Hxx(t-1,:);
        end
        u_t = Huu(t,:);
        z_t = z(t,:);
        if size(eps,2)>1
            eps_t = eps(:,t);
        else
            eps_t = eps(:,1);
        end
        [Hxx(t,:),res,iter,fval]=fsolution(Ssolver,fname,x0_t,xl_t,u_t,z_t,theta,eps_t,Nmaxit,GSmaxit,SStoler,disp_err,t,0);
    end
    
    % for t=1:(T-S+1) %mean
    %     for i=1:N
    %         u_mean(t,i)=mean(Hu(t,i,:));
    %     end
    % end
    % for t=1:(T-S+1) %mean
    %     for i=1:n
    %         x_mean(t,i)=mean(HxOffspring(t,i,:));
    %     end
    % end
    %
    % for j=1:N
    %    u_norm(j) = 0;
    %    for p=1:P
    %        u_norm(j) =  u_norm(j) + norm(Hu(:,j,p)-Huu(:,j));
    %    end
    % end
    %     bestv=min(fitC(it+1,:,:)); %find the best solution in the final population
    %     position=find(fitC(it+1,:,:)==bestv);
    %     Hux=Hu(:,1:T-S+1,position);
    %     res=fitC(it+1,position)';
    %     resultfit(MCcounter,:)=fitC(it+1,:);
    %     resultHu(:,:,MCcounter)=Hux;
    %
    %     fitC=fitStart;
    
    %end % end of all MCruns
end

%% Plot the evolution in each iteration
for bestS=1:it
    BestS1(bestS)=min(fitC(bestS,:));
end
BestS=BestS1(1,:); %single solution
% plot(BestS);
% box('on');
% set(gcf,'color','white');

%save newResultsLargeVersion_de -double
