function [G,g] = feedback_loop(fname,x,xSm1,u,z,theta,Covtheta,Covsyst,x_target,u_target,Wxx,Wux,Wuu,alpha,eps,stochpar,tt,t)
%calculate feedback rules (G,g) for finding optimal policy

n=size(x,2);
m=size(u,2);
l=size(z,2);
p=size(theta,1);

Wxu=Wux';

H = zeros(n, n);
hx = zeros(n,1);

hc = 0;
hs = 0;
hp = 0;

G = zeros(m,n,tt);
g = zeros(m,tt);

%double timBegin;
while (tt >= t)
    
    % /*** Linearize system equation ***/
    A = zeros(n, n);
    B = zeros(n, m);
    C = zeros(n, l);
    phi = zeros(n,1);
    x0t = x(tt,:);
    u0t = u(tt,:);
    z0t = z(tt,:);
    eps = zeros(n,1);
    if tt==1                       
        x0tm1 = xSm1;
    else
        x0tm1 = x(tt-1,:);                    
    end
    [A,B,C,phi,Da,Db,Dc,Dphi,COVxi] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,Covsyst,eps,stochpar);
  
    %calculating stochastic influence. 
    UpsAKA = zeros(n, n);
    UpsBKA = zeros(m, n);
    UpsBKB = zeros(m, m);
    UpsAKC = zeros(n, l);
    UpsBKC = zeros(m, l);
    UpsCKC = zeros(l, l);
    UpsAKphi = zeros(n,1);
    UpsBKphi = zeros(m,1);
    UpsCKphi = zeros(l,1);
    UpsphiKphi = 0;
    
    if (stochpar~=0) %calculate Ups matrices analyticaly    
        K = Wxx*alpha^(tt-1) + H;
        UpsAKA = upsBKA(K,Da,Covtheta,Da);
        UpsBKA = upsBKA(K,Db,Covtheta,Da);
        UpsBKB = upsBKA(K,Db,Covtheta,Db);
        UpsAKC = upsBKA(K,Da,Covtheta,Dc);
        UpsBKC = upsBKA(K,Db,Covtheta,Dc);
        UpsCKC = upsBKA(K,Dc,Covtheta,Dc);
        UpsAKphi = upsBKA(K,Da,Covtheta,Dphi);
        UpsBKphi = upsBKA(K,Db,Covtheta,Dphi);
        UpsCKphi = upsBKA(K,Dc,Covtheta,Dphi);
        UpsphiKphi = upsBKA(K,Dphi,Covtheta,Dphi);
    end

    % Convert objective function from tracking format to general
    % quadratic format - step (2biv) in paper*/
    xtarget = x_target(tt,:);
    utarget = u_target(tt,:);
    wx = -Wxx*alpha^(tt-1)*xtarget' - Wxu*alpha^(tt-1)*utarget';
    wu = -Wux*alpha^(tt-1)*xtarget' - Wuu*alpha^(tt-1)*utarget';
    wc = 0.5*xtarget*Wxx*alpha^(tt-1)*xtarget' + utarget*Wux*alpha^(tt-1)*xtarget' + 0.5*utarget*Wuu*alpha^(tt-1)*utarget';

    % Compute parameters of feedback rule and expected minimal loss
    % function   
    K = Wxx*alpha^(tt-1) + H;
    kx = wx + hx;

    AK = A'*K;
    BK = B'*K;
    CK = C'*K;
    Cz = C*z0t';

    LAMxx = UpsAKA + AK*A;
    LAMux = UpsBKA + BK*A + Wux*alpha^(tt-1)*A;
    LAMxu = LAMux';
    LAMuu = UpsBKB + BK*B + 2*B'*Wxu*alpha^(tt-1) + Wuu*alpha^(tt-1);

    LAMx = AK*Cz + UpsAKC*z0t' + AK*phi + UpsAKphi + A'*kx;
    LAMu = BK*Cz + UpsBKC*z0t' + BK*phi + UpsBKphi + B'*kx + Wux*alpha^(tt-1)*Cz + Wux*alpha^(tt-1)*phi + wu;   
    LAMz = CK*phi + UpsCKphi + C'*kx;
    LAMc = 0.5*Cz'*K*Cz + 0.5*z0t*UpsCKC*z0t' + wc + hc;
    
    LAMs = 0.5*trace(K*COVxi) + hs;
    LAMp = 0.5*phi'*K*phi + 0.5*UpsphiKphi + phi'*kx + hp; 

    LAMuuINV = inv(LAMuu);

    % Calculate G,g
    G(:,:,tt) = -LAMuuINV*LAMux;
    g(:,tt) = -LAMuuINV*LAMu;

    % Compute the parameters of the function of minimal expected
    % accumulated loss step (2bvii) in paper 
    H = LAMxx - LAMxu*LAMuuINV*LAMux;
    hx = LAMx - LAMxu*LAMuuINV*LAMu;
    hc = LAMc - 0.5 * LAMu'*LAMuuINV*LAMu;
    hs = LAMs;
    hp = LAMp + z0t*LAMz;

    tt = tt - 1;
end