function [A,B,c] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,eps)
%function [A,B,C,phi,Da,Db,Dc,Dphi,COVxi] = linearization(fname,x0t,x0tm1,u0t,z0t,theta,COVsyst,eps,stochpar)
%Compute the expected values of the paramters of the linearized system of
%equations

%Take care of '0'-values by calculating the step h!!!

n=size(x0t,2);
m=size(u0t,2);
%l=size(z0t,2);
%p=size(theta,1);

%Fxtm1, partial derivative with respect to xtm1
fxtm1=zeros(n,n);
hstep = 0.00001.*x0tm1;
hstep(hstep==0)=0.1;%hstep(hstep==0)=0.01;
hstep = abs(hstep);
hdiag = diag(hstep);
for i = 1:n
    h = hdiag(i,:);
    fxtm1(:,i) = (feval(fname,x0t,x0tm1+h,u0t,z0t,theta,eps,0) - feval(fname,x0t,x0tm1-h,u0t,z0t,theta,eps,0))./(2*hstep(i));
end

%Fxt, partial derivative with respect to xt
fxt=zeros(n,n);
hstep = 0.00001.*x0t;
hstep(hstep==0)=0.1;
hstep = abs(hstep);
hdiag = diag(hstep);
for i = 1:n
    h = hdiag(i,:);
    fxt(:,i) = (feval(fname,x0t+h,x0tm1,u0t,z0t,theta,eps,0) - feval(fname,x0t-h,x0tm1,u0t,z0t,theta,eps,0))./(2*hstep(i));
end

%Fut, partial derivative with respect to ut
fut=zeros(n,m);
hstep = 0.00001.*u0t;
hstep(hstep==0)=0.1;
hstep = abs(hstep);
hdiag = diag(hstep);
for i = 1:m
    h = hdiag(i,:);
    fut(:,i) = (feval(fname,x0t,x0tm1,u0t+h,z0t,theta,eps,0) - feval(fname,x0t,x0tm1,u0t-h,z0t,theta,eps,0))./(2*hstep(i));
end

% %Fzt, partial derivative with respect to zt
% fzt=zeros(n,l);
% hstep = 0.00001.*z0t;
% hstep(hstep==0)=0.1;
% hstep = abs(hstep);
% hdiag = diag(hstep);
% for i = 1:l
%     h = hdiag(i,:);
%     fzt(:,i) = (feval(fname,x0t,x0tm1,u0t,z0t+h,theta,eps,0) - feval(fname,x0t,x0tm1,u0t,z0t-h,theta,eps,0))./(2*hstep(i));
% end

% if (stochpar~=0) %second partial derivatives
%     
%     fthetat=zeros(n,p);
%     hthetastep = 0.00001.*theta;
%     hthetastep(hthetastep==0)=0.1;
%     hthetastep = abs(hthetastep);
%     hthetadiag = diag(hthetastep);
%     for i = 1:p
%         htheta = hthetadiag(:,i);
%         fthetat(:,i) = (feval(fname,x0t,x0tm1,u0t,z0t,theta+htheta,eps,0) - feval(fname,x0t,x0tm1,u0t,z0t,theta-htheta,eps,0))./(2*hthetastep(i));
%     end
%     
%     hxtm1=zeros(n*p,n);
%     hstep = 0.00001.*x0tm1;
%     hstep(hstep==0)=0.1;%hstep(hstep==0)=0.01;
%     hstep = abs(hstep);
%     hdiag = diag(hstep);
%     for i = 1:n
%         h = hdiag(i,:);
%         for j=1:p
%             htheta = hthetadiag(:,j);
%             htemp = ((feval(fname,x0t,x0tm1+h,u0t,z0t,theta+htheta,eps,0) - feval(fname,x0t,x0tm1-h,u0t,z0t,theta+htheta,eps,0))-(feval(fname,x0t,x0tm1+h,u0t,z0t,theta-htheta,eps,0) - feval(fname,x0t,x0tm1-h,u0t,z0t,theta-htheta,eps,0)))./(4*hstep(i)*hthetastep(j));
%             for k=1:n
%                 hxtm1(((k-1)*p+j),i) = htemp(k);
%             end
%         end
%     end
%     
%     hxt=zeros(n*p,n);
%     hstep = 0.00001.*x0t;
%     hstep(hstep==0)=0.1;%hstep(hstep==0)=0.01;
%     hstep = abs(hstep);
%     hdiag = diag(hstep);
%     for i = 1:n
%         h = hdiag(i,:);
%         for j=1:p
%             htheta = hthetadiag(:,j);
%             htemp = ((feval(fname,x0t+h,x0tm1,u0t,z0t,theta+htheta,eps,0) - feval(fname,x0t-h,x0tm1,u0t,z0t,theta+htheta,eps,0))-(feval(fname,x0t+h,x0tm1,u0t,z0t,theta-htheta,eps,0) - feval(fname,x0t-h,x0tm1,u0t,z0t,theta-htheta,eps,0)))./(4*hstep(i)*hthetastep(j));
%             for k=1:n
%                 hxt(((k-1)*p+j),i) = htemp(k);
%             end
%         end
%     end
%     
%     hut=zeros(n*p,m);
%     hstep = 0.00001.*u0t;
%     hstep(hstep==0)=0.1;%hstep(hstep==0)=0.01;
%     hstep = abs(hstep);
%     hdiag = diag(hstep);
%     for i = 1:m
%         h = hdiag(i,:);
%         for j=1:p
%             htheta = hthetadiag(:,j);
%             htemp = ((feval(fname,x0t,x0tm1,u0t+h,z0t,theta+htheta,eps,0) - feval(fname,x0t,x0tm1,u0t-h,z0t,theta+htheta,eps,0))-(feval(fname,x0t,x0tm1,u0t+h,z0t,theta-htheta,eps,0) - feval(fname,x0t,x0tm1,u0t-h,z0t,theta-htheta,eps,0)))./(4*hstep(i)*hthetastep(j));
%             for k=1:n
%                 hut(((k-1)*p+j),i) = htemp(k);
%             end
%         end
%     end
%     
% %     hzt=zeros(n*p,l);
% %     hstep = 0.00001.*z0t;
% %     hstep(hstep==0)=0.1;%hstep(hstep==0)=0.01;
% %     hstep = abs(hstep);
% %     hdiag = diag(hstep);
% %     for i = 1:l
% %         h = hdiag(i,:);
% %         for j=1:p
% %             htheta = hthetadiag(:,j);
% %             htemp = ((feval(fname,x0t,x0tm1,u0t,z0t+h,theta+htheta,eps,0) - feval(fname,x0t,x0tm1,u0t,z0t-h,theta+htheta,eps,0))-(feval(fname,x0t,x0tm1,u0t,z0t+h,theta-htheta,eps,0) - feval(fname,x0t,x0tm1,u0t,z0t-h,theta-htheta,eps,0)))./(4*hstep(i)*hthetastep(j));
% %             for k=1:n
% %                 hzt(((k-1)*p+j),i) = htemp(k);
% %             end
% %         end
% %     end
% end

inverse = inv(eye(n) - fxt);        
A = inverse*fxtm1;
B = inverse*fut;
%C = inverse*fzt;
%phi = x0t' - A*x0tm1' - B*u0t' - C*z0t';
c = x0t' - A*x0tm1' - B*u0t';

%COVxi = inverse*COVsyst*inverse';

% %second derivatives
% Da = zeros(n*p,n);
% Db = zeros(n*p,m);
% % Dc = zeros(n*p,l);
% % Dphi = zeros(n*p,1);
% Dc = zeros(n*p,1);
% if (stochpar~=0)
%     Da=kron(inverse,eye(p))*(hxt*A+hxtm1);
%     Db=kron(inverse,eye(p))*(hxt*B+hut);
% %     Dc=kron(inverse,eye(p))*(hxt*C+hzt);
% %     Dphi=reshape((inverse*fthetat)',n*p,1)-Da*x0tm1'-Db*u0t'-Dc*z0t';
%     Dc=reshape((inverse*fthetat)',n*p,1)-Da*x0tm1'-Db*u0t';
% end