clear; close all; clc;
format long

%%Input data

fname='mumod1';
%fname='testgame';
%scenario='mumod1_2014_08_unexp';
%scenario='mumod1_ineq_and_asym';
scenario='mumod1_ineq_W_RE_1';
%scenario='mumod1_OL';

%load(fname)
load(scenario)

n = size(xSm1,2); %number of state variables
m = size(u_start,2); %number of control variables
l = size(z,2); %number of exogenous variables
p = size(theta,1); %number of parameters
nv = (n + m) * (T - s + 1); %total number of varibles (for convergence)
NLtoler = 0.00001; %absolut tolerance for non-linearity loop
NLmaxit = 30; %number of maximal iterations for non-linearity loop
Ssolver = 'gauss-seidel'; %which system solver should be used: newton, gauss-seidel or matlab fsolve
SStoler = 0.00001; %absolut tolerance for system solver
Nmaxit = 10; %number of maximal iterations for system solver (newton line search)
GSmaxit = 50; %number of maximal iterations for system solver (gauss-seidel)
disp_err = 'error'; %if 'on' display exit_flag for fsolve; 'error' display errors only; 'off' otherwise
stochpar=0; %0 - deterministic, 1- stochastic (analytical), 2 - stochastic (numerical)
DEtoler = 0.00001; %absolut tolerance for system solver
numberMC_DE = 100; %number of Monte Carlo runs
if(exist('alpha','var')==0)
    alpha = 1; %discount rate for weigthing matrix W!!! e.g. 1.03 or 1.02
end
Jkind = 0; %different ways for calculating J; 0 - standard, 1 - asymmetric, 
beta=sqrt(0.1); %penalty weigth for the values inside the thresholds (<= 1)
if(exist('tt_shock','var')==0)
    tt_shock = 0; %time period(s) when the shock occurs
    z_shock = 0; %exogenous variables incl. shock
end


%%Step 0: calculating tentative path for x
%time_start = cputime; 
u = u_start;
x = zeros(T-s+1,n);
%eps = zeros(n,1);
for t=1:T-s+1 %calculate the initial solution path
    if t==1
        x0_t = xSm1;                        % Starting guess
        xl_t = xSm1;
    else
        x0_t = x(t-1,:);                    % Starting guess
        xl_t = x(t-1,:);
    end
    u_t = u(t,:);
    z_t = z(t,:);
    
    [x(t,:),res,iter,fval]=fsolution(Ssolver,fname,x0_t,xl_t,u_t,z_t,theta,eps,Nmaxit,GSmaxit,SStoler,disp_err,t,0);
end
%tentative_time = cputime-time_start;
x_start = x; %initial values for states

% put together all the individual matrices for targets and weights 
Wxx=zeros(size(Wxx1,1),size(Wxx1,2),NP);
Wtxx=zeros(size(Wxx1,1),size(Wxx1,2),NP);
Wuu=zeros(size(Wuu1,1),size(Wuu1,2),NP);
Wtuu=zeros(size(Wuu1,1),size(Wuu1,2),NP);
x_target = zeros(size(x_target1,1),size(x_target1,2),NP);
u_target = zeros(size(u_target1,1),size(u_target1,2),NP);
for i=1:NP
    I = num2str(i);
    eval(['Wxx(:,:,i) = Wxx' I ';']);
    eval(['Wuu(:,:,i) = Wuu' I ';']);
    eval(['Wtxx(:,:,i) = Wtxx' I ';']);
    eval(['Wtuu(:,:,i) = Wtuu' I ';']);
    eval(['x_target(:,:,i) = x_target' I ';']);
    eval(['u_target(:,:,i) = u_target' I ';']);
end

if(exist('TE','var')==0) %especial cases (for example higher weights Wtxx in the last period)
    TE = 0;
end

%check threshold_input: notice in case if target=0; thresholds are
%considered to be absolute deviations
if(exist('threshold_input','var')==0)
    threshold_input = 0;
end

J_simulat = calculateJ(NP,x_start,u_start,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,0,0,0,0,0);
%J_simulat1 = calculateJ(NP,x_start(1:10,:),u_start(1:10,:),x_target(1:10,:,:),u_target(1:10,:,:),Wxx,Wuu,Wtxx,Wtuu,alpha,10,s,TE,0,0,0,0,0);
%J_simulat2 = calculateJ(NP,x_start(11:30,:),u_start(11:30,:),x_target(11:30,:,:),u_target(11:30,:,:),Wxx,Wuu,Wtxx,Wtuu,alpha,T,11,TE,0,0,0,0,0);
%J_simulat_sum = J_simulat1+J_simulat2;

%%solution methods: classical: 'Pareto', 'NashOL', 'NashFB', 'StackOL', 'StackFB', 'all'
SolutionMethod = {'Pareto' , 'NashFB', 'NashOL'}; 
%time_start = cputime;

%% Pareto solution 
tic
if sum(strcmpi(SolutionMethod, 'Pareto') + strcmpi(SolutionMethod, 'all')) > 0
    if (exist('muPareto','var')==0)
        for player=1:NP
            muPareto(player) = 1/NP;
        end
    end
    [x_Pareto,u_Pareto,J_Pareto,convergNL_P,iteratNL_P] = NLloop_Pareto(fname,NP,NU,x_start,xSm1,u_start,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,muPareto,eps,stochpar,T,s,s,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
    disp(['Summary: Pareto strategy; J(p1, p2,...,sum(J)) = ' num2str(J_Pareto) '; #iterations = ' num2str(iteratNL_P) '; convergence = ' num2str(convergNL_P) ' of ' num2str(nv)])

    if sum(tt_shock) > 0
        z_orig=z; %save the original z
        Wxx_orig=Wxx; %save the original Wxx
        %Wxx=Wxx_shock; %after shock use the changed weights
        for i = 1:T
            S_subgame = i; %starting point of the 'remaining' problem
            if tt_shock(i)==1 %if an unexpected shock occurs in t only, recalculate remaining periods with new data
                z(i,:) = z_shock(i,:); %new data with shock
                %[x_Pareto(i:T,:),u_Pareto(i:T,:),J_Pareto,convergNL_P,iteratNL_P] = NLloop_Pareto(fname,NP,NU,x_start,xSm1,u_start,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,muPareto,eps,stochpar,T,s,S_subgame,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
                [x_Pareto,u_Pareto,J_Pareto,convergNL_P,iteratNL_P] = NLloop_Pareto(fname,NP,NU,x_Pareto,xSm1,u_Pareto,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,muPareto,eps,stochpar,T,s,S_subgame,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
            elseif tt_shock(i) == 2 %if an unexpected shock occurs in t with known impacts for the remaining periods, recalculate remaining periods with new data
                z(i:T,:) = z_shock(i:T,:); %new data with shock
                [x_Pareto,u_Pareto,J_Pareto,convergNL_P,iteratNL_P] = NLloop_Pareto(fname,NP,NU,x_Pareto,xSm1,u_Pareto,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,muPareto,eps,stochpar,T,s,S_subgame,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
                [x_tt,u_tt,J_tt,convergNL_tt,iteratNL_tt] = NLloop_Pareto(fname,NP,NU,x_Pareto,xSm1,u_Pareto,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,muPareto,eps,stochpar,T,s,S_subgame,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
            end
            J_Pareto = calculateJ(NP,x_Pareto,u_Pareto,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,0,0,0,0,0);
        end
        z=z_orig; %use original z for other solutions
        Wxx=Wxx_orig; %use original Wxx for other solutions
    end
end

%% Nash feedback solution 
if sum(strcmpi(SolutionMethod, 'NashFB') + strcmpi(SolutionMethod, 'all')) > 0
    [x_NashFB,u_NashFB,J_NashFB,convergNL_NFB,iteratNL_NFB] = NLloop_NashFB(fname,NP,NU,x_start,xSm1,u_start,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,s,s,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
    disp(['Summary: NashFB strategy; J(p1, p2,...,sum(J)) = ' num2str(J_NashFB) '; #iterations = ' num2str(iteratNL_NFB) '; convergence = ' num2str(convergNL_NFB) ' of ' num2str(nv)])

    if sum(tt_shock) > 0
        z_orig=z; %save the original z
        Wxx_orig=Wxx; %save the original Wxx
        %Wxx=Wxx_shock; %after shock use the changed weights
        for i = 1:T
            if tt_shock(i)==1 %if an unexpected shock occurs in t only, recalculate remaining periods with new data
                S_subgame = i; %starting point of the 'remaining' problem
                z(i,:) = z_shock(i,:); %new data with shock
                [x_NashFB,u_NashFB,J_NashFB,convergNL_NFB,iteratNL_NFB] = NLloop_NashFB(fname,NP,NU,x_NashFB,xSm1,u_NashFB,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,s,S_subgame,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
                J_NashFB = calculateJ(NP,x_NashFB,u_NashFB,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,0,0,0,0,0);
            elseif tt_shock(i) == 2 %if an unexpected shock occurs in t with known impacts for the remaining periods, recalculate remaining periods with new data
                S_subgame = i; %starting point of the 'remaining' problem
                z(i:T,:) = z_shock(i:T,:); %new data with shock
                [x_NashFB,u_NashFB,J_NashFB,convergNL_NFB,iteratNL_NFB] = NLloop_NashFB(fname,NP,NU,x_NashFB,xSm1,u_NashFB,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,s,S_subgame,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
                J_NashFB = calculateJ(NP,x_NashFB,u_NashFB,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,0,0,0,0,0);
            end
        end
        z=z_orig; %use original z for other solutions
        Wxx=Wxx_orig; %use original Wxx for other solutions
    end
end


%% Nash OL solution 
if sum(strcmpi(SolutionMethod, 'NashOL') + strcmpi(SolutionMethod, 'all')) > 0
    [x_NashOL,u_NashOL,J_NashOL,convergNL_NOL,iteratNL_NOL] = NLloop_NashOL(fname,NP,NU,x_start,xSm1,u_start,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,s,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
end

%% Stackelberg feedback solution 
if sum(strcmpi(SolutionMethod, 'StackFB') + strcmpi(SolutionMethod, 'all')) > 0
    [x_StackFB,u_StackFB,J_StackFB,convergNL_SFB,iteratNL_SFB] = NLloop_StackFB(fname,NP,NU,x_start,xSm1,u_start,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,s,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
end


%% Stackelberg open-loop solution 
if sum(strcmpi(SolutionMethod, 'StackOL') + strcmpi(SolutionMethod, 'all')) > 0
    [x_StackOL,u_StackOL,J_StackOL,convergNL_SOL,iteratNL_SOL] = NLloop_StackOL(fname,NP,NU,x_start,xSm1,u_start,z,theta,x_target,u_target,Wxx,Wtxx,Wuu,Wtuu,alpha,eps,stochpar,T,s,TE,NLmaxit,NLtoler,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,0);
end

toc %total_time = cputime-time_start;

%% report convergence results
disp(['Summary: simulation; J(p1, p2,...,sum(J)) = ' num2str(J_simulat)])
if sum(strcmpi(SolutionMethod, 'Pareto') + strcmpi(SolutionMethod, 'all')) > 0
    disp(['Summary: Pareto strategy; J(p1, p2,...,sum(J)) = ' num2str(J_Pareto) '; #iterations = ' num2str(iteratNL_P) '; convergence = ' num2str(convergNL_P) ' of ' num2str(nv)])
end
if sum(strcmpi(SolutionMethod, 'NashFB') + strcmpi(SolutionMethod, 'all')) > 0
    disp(['Summary: NashFB strategy; J(p1, p2,...,sum(J)) = ' num2str(J_NashFB) '; #iterations = ' num2str(iteratNL_NFB) '; convergence = ' num2str(convergNL_NFB) ' of ' num2str(nv)])
end
if sum(strcmpi(SolutionMethod, 'NashOL') + strcmpi(SolutionMethod, 'all')) > 0
    disp(['Summary: NashOL strategy; J(p1, p2,...,sum(J)) = ' num2str(J_NashOL) '; #iterations = ' num2str(iteratNL_NOL) '; convergence = ' num2str(convergNL_NOL) ' of ' num2str(nv)])
end
if sum(strcmpi(SolutionMethod, 'StackFB') + strcmpi(SolutionMethod, 'all')) > 0
    disp(['Summary: StackFB strategy; J(p1, p2,...,sum(J)) = ' num2str(J_StackFB) '; #iterations = ' num2str(iteratNL_SFB) '; convergence = ' num2str(convergNL_SFB) ' of ' num2str(nv)])
end
if sum(strcmpi(SolutionMethod, 'StackOL') + strcmpi(SolutionMethod, 'all')) > 0
    disp(['Summary: StackOL strategy; J(p1, p2,...,sum(J)) = ' num2str(J_StackOL) '; #iterations = ' num2str(iteratNL_SOL) '; convergence = ' num2str(convergNL_SOL) ' of ' num2str(nv)])
end
    


%% heuristic method (differential evolution)
%solution methods: DE, PSO
SolutionMethod_DE = {'DE_NashOL'};% 'DE_NashFB', 'DE_NashOL', 'DE_Pareto', 'DE_all'
Modul='iterative'; %iterative, 
ineq_constr=1; %=1 if inequality constraints should be considered. adjust calculateJ_ineq accordingly

if strncmpi(SolutionMethod_DE, 'DE',2) == 1
    maxiter=100;
    %uDE=u_NashFB;
    ucount=1;
    for player=1:NP
        [F_DE, CR_DE, Pop_DE(player), DEmaxit, DEconvergCrit, neigDE] = calibrated_input(fname,Jkind,u_start,u_NashFB,'DE',player);
        uf(player)=ucount;%count for the first instrument u of player
        ul(player)=uf(player)+NU(player)-1;%count for the last instrument u of player
        ucount=ul(player)+1;
    end
    
    Restarts=10;
    for restart=1:Restarts
        iteration=1; converg_DE = 0;
        xDE=x_start;
        uDE=u_start;
        restart
        
        %Pareto solution
        tic
        if sum(strcmpi(SolutionMethod_DE, 'DE_Pareto') + strcmpi(SolutionMethod_DE, 'DE_all')) > 0
            Wxx_P=zeros(size(Wxx1,1),size(Wxx1,2),NP);
            Wuu_P=zeros(size(Wuu1,1),size(Wuu1,2),NP);
            for i=1:NP
                I = num2str(i);
                eval(['Wxx_P(:,:,i) = Wxx' I '*muPareto(i)*NP;']);
                eval(['Wuu_P(:,:,i) = Wuu' I '*muPareto(i)*NP;']);
                eval(['Wtxx_P(:,:,i) = Wtxx' I '*muPareto(i)*NP;']);
                eval(['Wtuu_P(:,:,i) = Wtuu' I '*muPareto(i)*NP;']);
            end
            
            uf(NP+1)=1; ul(NP+1)=size(u_start,2);
            player=NP+1;
            if ineq_constr==1
                [xDE_P,uDE_P,J_DE_P,UDE_P,bestJ_P,BestS_P] = de_ineq(fname,x_start,xSm1,uDE,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,s,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx_P,Wuu_P,Wtxx_P,Wtuu_P,TE,alpha,F_DE,CR_DE,Pop_DE(player-1),Jkind,threshold_input,beta);
            else
                [xDE_P,uDE_P,J_DE_P,UDE_P,bestJ_P,BestS_P] = de(fname,x_start,xSm1,uDE,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,s,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx_P,Wuu_P,Wtxx_P,Wtuu_P,TE,alpha,F_DE,CR_DE,Pop_DE(player-1),Jkind,threshold_input,beta);
            end
            
            uDE_best_Pareto(:,:,restart)=uDE_P;
            xDE_best_Pareto(:,:,restart)=xDE_P;
            if ineq_constr==1
                J_DE_final_Pareto(restart,:)=calculateJ_ineq(NP,xDE_P,uDE_P,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,Jkind,threshold_input,beta,0,0);
            else
                J_DE_final_Pareto(restart,:)=calculateJ(NP,xDE_P,uDE_P,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,Jkind,threshold_input,beta,0,0);
            end
            %position_best_Pareto=find(sum(J_DE_final_Pareto,2)==min(sum(J_DE_final_Pareto,2)));
            %uDE_best_Pareto_minJ=uDE_best_Pareto(:,:,position_best_Pareto);
            compare_J_Pareto(restart,:)=J_DE_final_Pareto(restart,:)-J_Pareto;
            compare_u_Pareto(:,:,restart)=u_Pareto-uDE_P;
        end
        toc
        
        %Nash solution
        tic
        if sum(strncmpi(SolutionMethod_DE, 'DE_Nash',7) + strcmpi(SolutionMethod_DE, 'DE_all')) > 0
            while iteration<maxiter && converg_DE==0
                uDE_old=uDE;
                for player=1:NP %player=NP:-1:1
                    if ineq_constr==1
                        [xDE,uDE,J_DE,UDE,bestJ,BestS] = de_ineq(fname,x_start,xSm1,uDE,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,s,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE(player),Jkind,threshold_input,beta);
                    else
                        [xDE,uDE,J_DE,UDE,bestJ,BestS] = de(fname,x_start,xSm1,uDE,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,s,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE(player),Jkind,threshold_input,beta);
                    end
                end
                %check the convergency
                udiff = abs((uDE-uDE_old)./uDE_old);
                if max(udiff) <= NLtoler
                    converg_DE = 1;
                end
                iteration=iteration+1;
            end
        end
        if sum(strcmpi(SolutionMethod_DE, 'DE_NashOL') + strcmpi(SolutionMethod_DE, 'DE_all')) > 0
            uDE_best_NashOL(:,:,restart)=uDE;
            xDE_best_NashOL(:,:,restart)=xDE;
            if ineq_constr==1
                J_DE_final_NashOL(restart,:)=calculateJ_ineq(NP,xDE,uDE,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,Jkind,threshold_input,beta,0,0);
            else
                J_DE_final_NashOL(restart,:)=calculateJ(NP,xDE,uDE,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,Jkind,threshold_input,beta,0,0);
            end
            compare_J_NashOL(restart,:)=J_DE_final_NashOL(restart,:)-J_NashOL;
            compare_u_NashOL(:,:,restart)=u_NashOL-uDE;
        end
        toc
        if sum(strcmpi(SolutionMethod_DE, 'DE_NashFB') + strcmpi(SolutionMethod_DE, 'DE_all')) > 0 %calculate NashFB
            for tt=s+1:T
                iteration=1; converg_DE = 0;
                uDE_best_NashFB(tt-1,:,restart)=uDE(1,:);
                xDE_best_NashFB(tt-1,:,restart)=xDE(1,:);
                x_target_s=x_target(tt:T,:,:);
                u_target_s=u_target(tt:T,:,:);
                z_s=z(tt:T,:);
                uDE=uDE(2:T-tt+2,:);
                xSm1_s=xDE(1,:);%oder neu berechnen f(u_DE)?
                while iteration<maxiter && converg_DE==0
                    uDE_old=uDE;
                    for player=1:NP %player=NP:-1:1
                        [xDE,uDE,J_DE,UDE,bestJ,BestS] = de(fname,x_start,xSm1_s,uDE,NP,player,uf,ul,z_s,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,tt,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target_s,u_target_s,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE(player),Jkind,threshold_input,beta);
                    end
                    %check the convergency
                    udiff = abs((uDE-uDE_old)./uDE_old);
                    if max(udiff) <= NLtoler
                        converg_DE = 1;
                    end
                    iteration=iteration+1;
                end
            end
            
            uDE_best_NashFB(T,:,restart)=uDE;
            xDE_best_NashFB(T,:,restart)=xDE;
            J_DE_final_NashFB(restart,:)=calculateJ(NP,xDE_best_NashFB(:,:,restart),uDE_best_NashFB(:,:,restart),x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,s,TE,Jkind,threshold_input,beta,0,0);
            compare_J_NashFB(restart,:)=J_DE_final_NashFB(restart,:)-J_NashFB;
            compare_u_NashFB(:,:,restart)=u_NashFB-uDE_best_NashFB(:,:,restart);
        end
        
        save_results=0;
        if save_results == 1
            save_date=['results_main_optgame_1restart_' num2str(fname) '_' num2str(datestr(now, 'yyyy_mm_dd HH_MM'))];
            save(save_date, '-double');
        end
    end
    save_results=0;
    if save_results == 1
        save_date=['results_main_optgame_all_restarts_' num2str(fname) '_' num2str(datestr(now, 'yyyy_mm_dd HH_MM'))];
        save(save_date, '-double');
    end
end


%% Plot results
plot_on = 1;
if plot_on == 1
    PlotType = 2; %1 - subplot of all results followed by step-by-step results; 2 - subplots only
    plotMethod = {'Pareto-DE', 'Nash-DE'}; %{'simulation', 'Pareto', 'Nash-FB', 'Nash-OL', 'Stack-FB', 'Stack-OL', 'Pareto-DE', 'Nash-DE', 'target'}
    %plotMethod = {'simulation', 'Pareto-DE', 'Nash-DE'}; %{'simulation', 'Pareto', 'Nash-FB', 'Nash-OL', 'Stack-FB', 'Stack-OL', 'Pareto-DE', 'Nash-DE', 'target'}
    plotMethod_title = {'Pareto_DE', 'Nash_DE'}; % 'simulation', 'Pareto', 'Nash', 'Pareto_DE', 'Nash_DE'
    numPM = size(plotMethod,2); %number of solution strategies which should be plotted
    
    if(exist('list_objectives','var')==0)
        list_objectives = zeros(1,n);
        list_objectives(1:16) = 1;
    end
    nov=sum(list_objectives); %number of state variables which should be plotted
    x_dataTotal = zeros(T-s+1,n,numPM);
    u_dataTotal = zeros(T-s+1,m,numPM);
    
    i=1;
    if sum(strcmpi(plotMethod, 'simulation')) > 0;
        x_dataTotal(:,:,i)=x_start;
        u_dataTotal(:,:,i)=u_start;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Pareto')) > 0;
        x_dataTotal(:,:,i)=x_Pareto;
        u_dataTotal(:,:,i)=u_Pareto;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Nash-FB')) > 0;
        x_dataTotal(:,:,i)=x_NashFB;
        u_dataTotal(:,:,i)=u_NashFB;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Nash-OL')) > 0;
        x_dataTotal(:,:,i)=x_NashOL;
        u_dataTotal(:,:,i)=u_NashOL;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Stack-FB')) > 0;
        x_dataTotal(:,:,i)=x_StackFB;
        u_dataTotal(:,:,i)=u_StackFB;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Stack-OL')) > 0;
        x_dataTotal(:,:,i)=x_StackOL;
        u_dataTotal(:,:,i)=u_StackOL;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Pareto-DE')) > 0;
        x_dataTotal(:,:,i)=xDE_P;
        u_dataTotal(:,:,i)=uDE_best_Pareto(:,:,restart);
        %u_dataTotal(:,:,i)=uDE_P;
        i=i+1;
    end
    if sum(strcmpi(plotMethod, 'Nash-DE')) > 0;
        x_dataTotal(:,:,i)=xDE;
        u_dataTotal(:,:,i)=uDE;
        i=i+1;
    end
    
    x_dataPlot = zeros(T-s+1,nov,numPM);
    j=1;
    for i=1:n
        if list_objectives(i)==1;
            x_dataPlot(:,j,:)=x_dataTotal(:,i,:);
            x_namesPlot(j)=x_names(i);
            j=j+1;
        end
    end
    
    %subplot_results(fname,x_dataPlot,u_dataTotal,x_namesPlot,u_names,plotMethod,numPM,list_objectives,PlotType);
    
    pcn=1; %plot control variable number 'pcn'
    psn=1; %plot state variable number 'psn'
    
    %plot_results(fname,x_dataPlot,u_dataTotal,x_namesPlot,u_names,plotMethod_title,numPM,pcn,psn);
    plot_results(fname,x_dataPlot,u_dataTotal,x_namesPlot,u_names,plotMethod_title,numPM,3,0);
    export_fig results/mumod1/g2_v2b.eps;
end

save_results=0;
if save_results == 1
    save_date=['results_main_optgame_' num2str(fname) '_' num2str(datestr(now, 'yyyy_mm_dd HH_MM'))];
    save(save_date, '-double'); 
end

