function [newtonx,fitn,iter] = newton(fname,x0_t,xl_t,u_t,z_t,theta,eps,maxitN,tol)
% Newton-Verfahren
% zur Loesung nichtlinearer Gleichungssysteme
% benutzt das Newton-Verfahren, um ausgehend von x0 einen Vektor
% newtonx zu finden, der das nichtlineare Gleichungssystem f(x) = b bis
% auf eine Genauigkeit tol loest. Dabei
% ist f die gegebene Modellfunktion.
%
% Fuer die Ausfuehrung sind folgende Parameter notwendig:
% 'fname' Name(Pfad) der Matlabfunktion zur Berechnung
% der Funktionwerte f(x) in der Fixpunktdarstellung 
% 'Dfname' Name(Pfad) der Matlabfunktion zur Berechnung
% der ersten Ableitungen f'(x)
% 'x' Matrix mit den exogenen Variablen
% 't' Zeitpunkt (fr verzgerte Variablen)
% 'x0' Startwert (Spaltenvektor)
% 'tol' absolute Fehlerschranke fuer das Residuum ||f(x)-b||
% 'maxitN' maximale Anzahl der Iterationen
%
% Ergebnisparameter sind:
% newtonx Ergebnisvektor (Spaltenvektor)
% fitn Euklidische Norm des berechneten Wertes ||f(x)-b||
% iter Anzahl der ausgefuehrten Iterationen
n=size(x0_t,2);
iter = 0; newtonx = x0_t;
fx = feval(fname,newtonx,xl_t,u_t,z_t,theta,eps,0);
fitn=norm(newtonx-fx);
fitn_old=fitn*2;
div=0;
% Iterationszyklus
while (fitn > tol) && (iter < maxitN) && (div<3) 
    % 3. Abbruchbedingung ist erfllt, wenn drei Schritte hintereinander
    % keine Verbesserung der Fitness stattgefunden hat
    fitn_old=fitn;
    % Dfx = feval(Dfname,x,t,newtonx,u,theta);
    %Fxt, partial derivative with respect to xt
    Dfx=zeros(n,n);
    hstep = 0.00001.*x0_t;
    hstep = abs(hstep);
    hdiag = diag(hstep);
    for i = 1:n
        h = hdiag(i,:);
        Dfx(:,i) = (feval(fname,x0_t+h,xl_t,u_t,z_t,theta,eps,0) - feval(fname,x0_t-h,xl_t,u_t,z_t,theta,eps,0))./(2*hstep(i));
        Dfx(i,i) = -1;
    end
    newtonx = (newtonx' - Dfx\(fx'-newtonx'))';
    fx = feval(fname,newtonx,xl_t,u_t,z_t,theta,eps,0);
    fitn=norm(newtonx-fx);
    if (fitn>=fitn_old)
        div=div+1;
    else
        div=0;
    end
    iter= iter + 1;
end % while