%% Differential Evolution function
% starting with u_DE,x_DE search for
% Huu (and indirectly for Hxx), which minimizes the optimum control problem.
% f is the system of (nonlinear) equations.
function [Hxx,Huu,fitC,fitJ,Jpso,BestSpso] = pso(fname,x_DE,xSm1,u_DE,z,theta,eps,tol,neigDE,numberMC,T,S,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,Jkind,threshold_input,beta,nP,nG,initV,delta,c1,c2,PM,NP,TE,DEmaxit,DEconvergCrit,numberMC_DE,F_DE,CR_DE,Pop_DE,uf,ul,NLtoler)

%% Inputs:
% 'fname' name(path) to the function for calculating f
% 'maxitH' maximal number of iterations
% tol an absolute tolerance limit for DE: norm(f)<tol means convergence
% 'c' constant for the size of population 'P=c*N' for DE (usually: 2 < c < 10)
% neigDE the range for the neighbourhood of start population for DE
% depends on the quality and absolute values of u0; 'good'
% u0 => small neigDE; 'small' x0 => large neigDE

% Output:
% Hxx results for the state varaibles
% Huu results for the control varaibles
% fitC FitnessValues

N = size(u_DE,2); %number of control variables
n = size(x_DE,2); %number of state variables
P=nP*N+1;   % size of population (+ 1 because of x0)
  % u is the 1. population member
Hx=zeros(T,n,P,nG);
Hu=zeros(T,N,P,nG);
Hu(1:T,1:N,1,1)=u_DE;
V=zeros(T,N,P,nG);
xstart=x_DE; %start tentative path for x
fitC=0;
PM=PM/(N*T)
% start MCsimulation
%tempP = randn(P,numberMC);
%for MCcounter=1:numberMC
%     for i =1:numberP
%         theta2(i,MCcounter)=theta(i) + 0.01*sqrt(covtheta(i,i))*tempP(i,MCcounter);
%     end

%% Generate start population, i.e. uniform values from the interval [u0(1-neigDE), u0(1+neigDE)].
temp = rand(T-S+1,N,P);


temp = rand(T-S+1,N,P);
for p=1:(P-1)
    for i=1:N
        for t=1:T-S+1
            if strncmpi(fname, 'mumod',5) == 1
                Hu(t,i,p+1,1)=neigDE(1,i) + (neigDE(2,i)-neigDE(1,i)).*temp(t,i,p);  % definition of absolute deviation from 0
            else
                Hu(t,i,p+1,1)=(1-neigDE(i))*Hu(t,i) + 2*neigDE(i)*Hu(t,i).*temp(t,i,p);  % definition of % deviation from u0
            end
        end
    end
end
V=initV*Hu;
%Huinitial=Hu;

% calculate paths for x of the start population
for p = 1:P %% random initial solution (using u)

    Hu_old=Hu(:,:,p,1);
    for player=1:NP %player=NP:-1:1
        [xDE,HuDE,J_DE,UDE,bestJ,BestS] = de(fname,x_DE,xSm1,Hu(:,:,p,1),NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,S,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE(player),Jkind,threshold_input,beta);
    end
    %check the convergency
    udiff(:,:,p) = abs((HuDE-Hu_old)./Hu_old);
    if max(max(udiff(:,:,p))) <= NLtoler
        converg_DE = 1;
    end

    fitC(1,p)=0;
    % fitness of u(p),x(p)
    fitC(1,p) = max(max(udiff(:,:,p)));
    Hx(:,:,p,1)=xDE;
    Hu(:,:,p,1)=HuDE;
    fitJ(:,p)=calculateJ(NP,xDE,HuDE,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);
end

P_best=Hu(:,:,:,1);
X_best=Hx(:,:,:,1);
F_best=fitC;
J_best=fitJ;

[G_best g_best]=min(fitC);
BestS=G_best;
BestSpso(1)=G_best;
%% Main body including the generation loop with creating offspring,
%% crossover and selecting
SStolerchange = 0;
converg_DE=0;
it=2;
while it < nG  && converg_DE==0
    %     if (it<=0.85*nG && SStolerchange == 0)
    %         SStoler = SStoler*10000;
    %         SStolerchange = 1;
    %     elseif (it >= nG/5 && SStolerchange == 1)
    %             SStoler = SStoler/1;
    %             SStolerchange = 2;
    %     elseif (it >= .85*nG &&SStolerchange == 2)
    %             SStoler = SStoler/10000;
    %             SStolerchange = 3;
    %     end

    for p = 1:P     %% create offspring for member p
        %APPLY PSO TO THE POPULATION OF SOLUTIONS
        
        Hu_old=Hu(:,:,p,it-1);
        
        Delta_V(:,:,p)=c1*rand*(P_best(:,:,p)-Hu(:,:,p,it-1))+c2*rand*(P_best(:,:,g_best)-Hu(:,:,p,it-1));
        V(:,:,p,it)=delta*V(:,:,p,it-1)+Delta_V(:,:,p);
        Hu(:,:,p,it)=Hu(:,:,p,it-1)+V(:,:,p,it);

        for t=1:T-S+1
            for j=1:N
                if rand<PM
                    %NumDig=ceil(log10(abs(Hu(t,j,p,it))));
                    %if  NumDig<=0
                        Hu(t,j,p,it)=Hu(t,j,p,it)+(2*round(rand)-1)*rand*Hu(t,j,p,it)/log(it);
                    %else
                        %Hu(t,j,p,it)=Hu(t,j,p,it)+(2*round(rand)-1)*9*10^( randi([abs(NumDig)-ceil(log10(abs(it)))-3 abs(NumDig)-ceil(log10(abs(it)))-1]));%9*10^( randi([abs(NumDig)-4 abs(NumDig)-3]));
                    %end
                end
            end
        end

        %Hu_old=Hu(:,:,p,it);
        for player=1:NP %player=NP:-1:1
            [xDE,HuDE,J_DE,UDE,bestJ,BestS] = de(fname,x_DE,xSm1,Hu_old,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,S,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE(player),Jkind,threshold_input,beta);
        end
        Hu_old2=HuDE;
        for player=NP:-1:1
            [xDE,HuDE,J_DE,UDE,bestJ,BestS] = de(fname,x_DE,xSm1,Hu_old2,NP,player,uf,ul,z,theta,eps,DEmaxit,DEconvergCrit,neigDE,numberMC_DE,T,S,Ssolver,GSmaxit,Nmaxit,SStoler,disp_err,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,TE,alpha,F_DE,CR_DE,Pop_DE(player),Jkind,threshold_input,beta);
        end
        %check the convergency
        udiff(:,:,p) = abs((HuDE-Hu_old2)./Hu_old2);
        fitC(it,p)= max(max(udiff(:,:,p)));
        Hx(:,:,p,it)=xDE;
        Hu(:,:,p,it)=HuDE;
        fitJ(:,it,p)= calculateJ(NP,xDE,HuDE,x_target,u_target,Wxx,Wuu,Wtxx,Wtuu,alpha,T,S,TE,0,0,0,0,0);      
        if max(max(udiff(:,:,p))) <= NLtoler
            converg_DE = 1;
        end

    end

    for p=1:P
        if fitC(it,p)<F_best(p)
            P_best(:,:,p)=Hu(:,:,p,it);
            X_best(:,:,p)=Hx(:,:,p,it);
            F_best(p)=fitC(it,p);
            J_best(:,p)=fitJ(:,it,p);
        end

        if fitC(it,p)<G_best
            g_best=p;
            G_best=fitC(it,p);
        end
    end
    BestSpso(it)=G_best
    J_best
    it=it+1;
end

bestv=min(F_best(:)); %find the best solution in the final population
position=find(F_best(:)==bestv);
if numel(position)>1
    position1=position(1);
else
    position1=position;
end
Hxx=X_best(:,:,position1);
Huu=P_best(:,:,position1);
Jpso=J_best(:,position1);

%     bestv=min(fitC(it+1,:,:)); %find the best solution in the final population
%     position=find(fitC(it+1,:,:)==bestv);
%     Hux=Hu(:,1:T-S+1,position);
%     res=fitC(it+1,position)';
%     resultfit(MCcounter,:)=fitC(it+1,:);
%     resultHu(:,:,MCcounter)=Hux;
%
%     fitC=fitStart;

%end % end of all MCruns


%% Plot the evolution in each iteration
plot(BestSpso);
box('on');
set(gcf,'color','white');


